#include <avr/io.h>
#include <avr/interrupt.h>  /* for sei() */
#include <util/delay.h>     /* for _delay_ms() */
#include <avr/pgmspace.h>   /* required by usbdrv.h */

#include "peri.h"
#include "usbdrv.h"

#define RQ_GET_SWITCH1     1
#define RQ_GET_SWITCH2     2
#define RQ_GET_SWITCH3     3
#define RQ_GET_SWITCH4     4
#define RQ_GET_SWITCH5     5
#define RQ_GET_SWITCH6     6
#define RQ_GET_SWITCH7     7
#define RQ_GET_SWITCH8     8
#define RQ_GET_SWITCH9     9
#define RQ_GET_SWITCH10     10
#define RQ_GET_SWITCH11     11
#define RQ_GET_SWITCH12     12
#define RQ_GET_SWITCH13     13

/* ------------------------------------------------------------------------- */
/* ----------------------------- USB interface ----------------------------- */
/* ------------------------------------------------------------------------- */
usbMsgLen_t usbFunctionSetup(uint8_t data[8])
{
    usbRequest_t *rq = (void *)data;

    /* declared as static so they stay valid when usbFunctionSetup returns */
  static uint8_t switch_state;

    if (rq->bRequest == RQ_GET_SWITCH1)
    {
        switch_state = IS_SWITCH_1();
        usbMsgPtr = &switch_state;
        return 1;
    }
    else if (rq->bRequest == RQ_GET_SWITCH2)
    {
        switch_state = IS_SWITCH_2()>>1;
        usbMsgPtr = &switch_state;
        return 1;
    }
    else if (rq->bRequest == RQ_GET_SWITCH3)
    {
        switch_state = IS_SWITCH_3()>>2;
        usbMsgPtr = &switch_state;
        return 1;
    }
    else if (rq->bRequest == RQ_GET_SWITCH4)
    {
        switch_state = IS_SWITCH_4()>>3;
        usbMsgPtr = &switch_state;
        return 1;
    }
    else if (rq->bRequest == RQ_GET_SWITCH5)
    {
        switch_state = IS_SWITCH_5()>>4;
        usbMsgPtr = &switch_state;
        return 1;
    }
    else if (rq->bRequest == RQ_GET_SWITCH6)
    {
        switch_state = IS_SWITCH_6()>>5;
        usbMsgPtr = &switch_state;
        return 1;
    }
    else if (rq->bRequest == RQ_GET_SWITCH7)
    {
        switch_state = IS_SWITCH_7();
        usbMsgPtr = &switch_state;
        return 1;
    }
    else if (rq->bRequest == RQ_GET_SWITCH8)
    {
        switch_state = IS_SWITCH_8()>>1;
        usbMsgPtr = &switch_state;
        return 1;
    }
    else if (rq->bRequest == RQ_GET_SWITCH9)
    {
        switch_state = IS_SWITCH_9()>>2;
        usbMsgPtr = &switch_state;
        return 1;
    }
    else if (rq->bRequest == RQ_GET_SWITCH10)
    {
        switch_state = IS_SWITCH_10()>>3;
        usbMsgPtr = &switch_state;
        return 1;
    }
    else if (rq->bRequest == RQ_GET_SWITCH11)
    {
        switch_state = IS_SWITCH_11()>>4;
        usbMsgPtr = &switch_state;
        return 1;
    }
    else if (rq->bRequest == RQ_GET_SWITCH12)
    {
        switch_state = IS_SWITCH_12()>>5;
        usbMsgPtr = &switch_state;
        return 1;
    }
    else if (rq->bRequest == RQ_GET_SWITCH13)
    {
        switch_state = IS_SWITCH_13();
        usbMsgPtr = &switch_state;
        return 1;
    }

    /* default for not implemented requests: return no data back to host */
    return 0;
}

/* ------------------------------------------------------------------------- */
int main(void)
{
    init_peripheral();

    usbInit();

    /* enforce re-enumeration, do this while interrupts are disabled! */
    usbDeviceDisconnect();
    _delay_ms(300);
    usbDeviceConnect();

    /* enable global interrupts */
    sei();

    /* main event loop */
    for(;;)
    {
        usbPoll();
    }

    return 0;
}

/* ------------------------------------------------------------------------- */
